/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.DolphinMainWindow.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef DOLPHINMAINWINDOWADAPTOR_H
#define DOLPHINMAINWINDOWADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "dolphinmainwindow.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.dolphin.MainWindow
 */
class MainWindowAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.dolphin.MainWindow")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.dolphin.MainWindow\">\n"
"    <signal name=\"settingsChanged\"/>\n"
"    <method name=\"openDirectories\">\n"
"      <arg direction=\"in\" type=\"as\" name=\"dirs\"/>\n"
"      <arg direction=\"in\" type=\"b\" name=\"splitView\"/>\n"
"    </method>\n"
"    <method name=\"openFiles\">\n"
"      <arg direction=\"in\" type=\"as\" name=\"files\"/>\n"
"      <arg direction=\"in\" type=\"b\" name=\"splitView\"/>\n"
"    </method>\n"
"    <method name=\"activateWindow\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"activationToken\"/>\n"
"    </method>\n"
"    <method name=\"isActiveWindow\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"    </method>\n"
"    <method name=\"isUrlOpen\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"url\"/>\n"
"    </method>\n"
"    <method name=\"isItemVisibleInAnyView\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"urlOfItem\"/>\n"
"    </method>\n"
"    <method name=\"pasteIntoFolder\"/>\n"
"    <method name=\"quit\"/>\n"
"    <method name=\"slotSplitViewChanged\"/>\n"
"    <method name=\"slotTabBarChanged\"/>\n"
"  </interface>\n"
        "")
public:
    MainWindowAdaptor(DolphinMainWindow *parent);
    ~MainWindowAdaptor() override;

    inline DolphinMainWindow *parent() const
    { return static_cast<DolphinMainWindow *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    void activateWindow(const QString &activationToken);
    bool isActiveWindow();
    bool isItemVisibleInAnyView(const QString &urlOfItem);
    bool isUrlOpen(const QString &url);
    void openDirectories(const QStringList &dirs, bool splitView);
    void openFiles(const QStringList &files, bool splitView);
    void pasteIntoFolder();
    void quit();
    void slotSplitViewChanged();
    void slotTabBarChanged();
Q_SIGNALS: // SIGNALS
    void settingsChanged();
};

#endif
