// This file is generated by kconfig_compiler_kf6 from dolphin_searchsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef SEARCHSETTINGS_H
#define SEARCHSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class SearchSettings : public KConfigSkeleton
{
  public:

    static SearchSettings *self();
    ~SearchSettings() override;

    /**
      Set Location
    */
    static
    void setLocation( const QString & v )
    {
      if (!self()->isLocationImmutable())
        self()->mLocation = v;
    }

    /**
      Get Location
    */
    static
    QString location()
    {
      return self()->mLocation;
    }

    /**
      Is Location Immutable
    */
    static
    bool isLocationImmutable()
    {
      return self()->isImmutable( QStringLiteral( "Location" ) );
    }

    /**
      Set What
    */
    static
    void setWhat( const QString & v )
    {
      if (!self()->isWhatImmutable())
        self()->mWhat = v;
    }

    /**
      Get What
    */
    static
    QString what()
    {
      return self()->mWhat;
    }

    /**
      Is What Immutable
    */
    static
    bool isWhatImmutable()
    {
      return self()->isImmutable( QStringLiteral( "What" ) );
    }

    /**
      Set SearchTool
    */
    static
    void setSearchTool( const QString & v )
    {
      if (!self()->isSearchToolImmutable())
        self()->mSearchTool = v;
    }

    /**
      Get SearchTool
    */
    static
    QString searchTool()
    {
      return self()->mSearchTool;
    }

    /**
      Is SearchTool Immutable
    */
    static
    bool isSearchToolImmutable()
    {
      return self()->isImmutable( QStringLiteral( "SearchTool" ) );
    }

  protected:
    SearchSettings();
    friend class SearchSettingsHelper;


    // Search
    QString mLocation;
    QString mWhat;
    QString mSearchTool;
};

#endif

