/****************************************************************************
** Meta object code from reading C++ file 'kitemlistheaderwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.04.1/src/kitemviews/private/kitemlistheaderwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kitemlistheaderwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21KItemListHeaderWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto KItemListHeaderWidget::qt_create_metaobjectdata<qt_meta_tag_ZN21KItemListHeaderWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItemListHeaderWidget",
        "columnWidthChanged",
        "",
        "role",
        "currentWidth",
        "previousWidth",
        "sidePaddingChanged",
        "leftPaddingWidth",
        "rightPaddingWidth",
        "columnWidthChangeFinished",
        "columnMoved",
        "currentIndex",
        "previousIndex",
        "sortOrderChanged",
        "Qt::SortOrder",
        "current",
        "previous",
        "sortRoleChanged",
        "columnUnHovered",
        "columnIndex",
        "columnHovered",
        "slotSortRoleChanged",
        "slotSortOrderChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'columnWidthChanged'
        QtMocHelpers::SignalData<void(const QByteArray &, qreal, qreal)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 }, { QMetaType::QReal, 4 }, { QMetaType::QReal, 5 },
        }}),
        // Signal 'sidePaddingChanged'
        QtMocHelpers::SignalData<void(qreal, qreal)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 7 }, { QMetaType::QReal, 8 },
        }}),
        // Signal 'columnWidthChangeFinished'
        QtMocHelpers::SignalData<void(const QByteArray &, qreal)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 }, { QMetaType::QReal, 4 },
        }}),
        // Signal 'columnMoved'
        QtMocHelpers::SignalData<void(const QByteArray &, int, int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'sortOrderChanged'
        QtMocHelpers::SignalData<void(Qt::SortOrder, Qt::SortOrder)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 14, 16 },
        }}),
        // Signal 'sortRoleChanged'
        QtMocHelpers::SignalData<void(const QByteArray &, const QByteArray &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 15 }, { QMetaType::QByteArray, 16 },
        }}),
        // Signal 'columnUnHovered'
        QtMocHelpers::SignalData<void(int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Signal 'columnHovered'
        QtMocHelpers::SignalData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'slotSortRoleChanged'
        QtMocHelpers::SlotData<void(const QByteArray &, const QByteArray &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 15 }, { QMetaType::QByteArray, 16 },
        }}),
        // Slot 'slotSortOrderChanged'
        QtMocHelpers::SlotData<void(Qt::SortOrder, Qt::SortOrder)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 14, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KItemListHeaderWidget, qt_meta_tag_ZN21KItemListHeaderWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItemListHeaderWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KItemListHeaderWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KItemListHeaderWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21KItemListHeaderWidgetE_t>.metaTypes,
    nullptr
} };

void KItemListHeaderWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KItemListHeaderWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->columnWidthChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3]))); break;
        case 1: _t->sidePaddingChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 2: _t->columnWidthChangeFinished((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 3: _t->columnMoved((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->sortOrderChanged((*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[2]))); break;
        case 5: _t->sortRoleChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 6: _t->columnUnHovered((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->columnHovered((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->slotSortRoleChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 9: _t->slotSortOrderChanged((*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KItemListHeaderWidget::*)(const QByteArray & , qreal , qreal )>(_a, &KItemListHeaderWidget::columnWidthChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListHeaderWidget::*)(qreal , qreal )>(_a, &KItemListHeaderWidget::sidePaddingChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListHeaderWidget::*)(const QByteArray & , qreal )>(_a, &KItemListHeaderWidget::columnWidthChangeFinished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListHeaderWidget::*)(const QByteArray & , int , int )>(_a, &KItemListHeaderWidget::columnMoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListHeaderWidget::*)(Qt::SortOrder , Qt::SortOrder )>(_a, &KItemListHeaderWidget::sortOrderChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListHeaderWidget::*)(const QByteArray & , const QByteArray & )>(_a, &KItemListHeaderWidget::sortRoleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListHeaderWidget::*)(int )>(_a, &KItemListHeaderWidget::columnUnHovered, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListHeaderWidget::*)(int )>(_a, &KItemListHeaderWidget::columnHovered, 7))
            return;
    }
}

const QMetaObject *KItemListHeaderWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KItemListHeaderWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KItemListHeaderWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsWidget::qt_metacast(_clname);
}

int KItemListHeaderWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void KItemListHeaderWidget::columnWidthChanged(const QByteArray & _t1, qreal _t2, qreal _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void KItemListHeaderWidget::sidePaddingChanged(qreal _t1, qreal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KItemListHeaderWidget::columnWidthChangeFinished(const QByteArray & _t1, qreal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void KItemListHeaderWidget::columnMoved(const QByteArray & _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void KItemListHeaderWidget::sortOrderChanged(Qt::SortOrder _t1, Qt::SortOrder _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void KItemListHeaderWidget::sortRoleChanged(const QByteArray & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void KItemListHeaderWidget::columnUnHovered(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void KItemListHeaderWidget::columnHovered(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
