/****************************************************************************
** Meta object code from reading C++ file 'dolphinpart_ext.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.04.1/src/dolphinpart_ext.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dolphinpart_ext.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27DolphinPartBrowserExtensionE_t {};
} // unnamed namespace

template <> constexpr inline auto DolphinPartBrowserExtension::qt_create_metaobjectdata<qt_meta_tag_ZN27DolphinPartBrowserExtensionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DolphinPartBrowserExtension",
        "cut",
        "",
        "copy",
        "paste",
        "pasteTo",
        "QUrl",
        "reparseConfiguration"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'cut'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copy'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'paste'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pasteTo'
        QtMocHelpers::SlotData<void(const QUrl &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'reparseConfiguration'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DolphinPartBrowserExtension, qt_meta_tag_ZN27DolphinPartBrowserExtensionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DolphinPartBrowserExtension::staticMetaObject = { {
    QMetaObject::SuperData::link<KParts::NavigationExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27DolphinPartBrowserExtensionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27DolphinPartBrowserExtensionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27DolphinPartBrowserExtensionE_t>.metaTypes,
    nullptr
} };

void DolphinPartBrowserExtension::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DolphinPartBrowserExtension *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cut(); break;
        case 1: _t->copy(); break;
        case 2: _t->paste(); break;
        case 3: _t->pasteTo((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->reparseConfiguration(); break;
        default: ;
        }
    }
}

const QMetaObject *DolphinPartBrowserExtension::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DolphinPartBrowserExtension::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27DolphinPartBrowserExtensionE_t>.strings))
        return static_cast<void*>(this);
    return KParts::NavigationExtension::qt_metacast(_clname);
}

int DolphinPartBrowserExtension::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::NavigationExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN28DolphinPartFileInfoExtensionE_t {};
} // unnamed namespace

template <> constexpr inline auto DolphinPartFileInfoExtension::qt_create_metaobjectdata<qt_meta_tag_ZN28DolphinPartFileInfoExtensionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DolphinPartFileInfoExtension"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DolphinPartFileInfoExtension, qt_meta_tag_ZN28DolphinPartFileInfoExtensionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DolphinPartFileInfoExtension::staticMetaObject = { {
    QMetaObject::SuperData::link<KParts::FileInfoExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28DolphinPartFileInfoExtensionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28DolphinPartFileInfoExtensionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28DolphinPartFileInfoExtensionE_t>.metaTypes,
    nullptr
} };

void DolphinPartFileInfoExtension::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DolphinPartFileInfoExtension *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *DolphinPartFileInfoExtension::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DolphinPartFileInfoExtension::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28DolphinPartFileInfoExtensionE_t>.strings))
        return static_cast<void*>(this);
    return KParts::FileInfoExtension::qt_metacast(_clname);
}

int DolphinPartFileInfoExtension::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::FileInfoExtension::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN33DolphinPartListingFilterExtensionE_t {};
} // unnamed namespace

template <> constexpr inline auto DolphinPartListingFilterExtension::qt_create_metaobjectdata<qt_meta_tag_ZN33DolphinPartListingFilterExtensionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DolphinPartListingFilterExtension"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DolphinPartListingFilterExtension, qt_meta_tag_ZN33DolphinPartListingFilterExtensionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DolphinPartListingFilterExtension::staticMetaObject = { {
    QMetaObject::SuperData::link<KParts::ListingFilterExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33DolphinPartListingFilterExtensionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33DolphinPartListingFilterExtensionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33DolphinPartListingFilterExtensionE_t>.metaTypes,
    nullptr
} };

void DolphinPartListingFilterExtension::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DolphinPartListingFilterExtension *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *DolphinPartListingFilterExtension::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DolphinPartListingFilterExtension::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33DolphinPartListingFilterExtensionE_t>.strings))
        return static_cast<void*>(this);
    return KParts::ListingFilterExtension::qt_metacast(_clname);
}

int DolphinPartListingFilterExtension::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::ListingFilterExtension::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN39DolphinPartListingNotificationExtensionE_t {};
} // unnamed namespace

template <> constexpr inline auto DolphinPartListingNotificationExtension::qt_create_metaobjectdata<qt_meta_tag_ZN39DolphinPartListingNotificationExtensionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DolphinPartListingNotificationExtension",
        "slotNewItems",
        "",
        "KFileItemList",
        "slotItemsDeleted"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotNewItems'
        QtMocHelpers::SlotData<void(const KFileItemList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotItemsDeleted'
        QtMocHelpers::SlotData<void(const KFileItemList &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DolphinPartListingNotificationExtension, qt_meta_tag_ZN39DolphinPartListingNotificationExtensionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DolphinPartListingNotificationExtension::staticMetaObject = { {
    QMetaObject::SuperData::link<KParts::ListingNotificationExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN39DolphinPartListingNotificationExtensionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN39DolphinPartListingNotificationExtensionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN39DolphinPartListingNotificationExtensionE_t>.metaTypes,
    nullptr
} };

void DolphinPartListingNotificationExtension::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DolphinPartListingNotificationExtension *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotNewItems((*reinterpret_cast<std::add_pointer_t<KFileItemList>>(_a[1]))); break;
        case 1: _t->slotItemsDeleted((*reinterpret_cast<std::add_pointer_t<KFileItemList>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *DolphinPartListingNotificationExtension::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DolphinPartListingNotificationExtension::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN39DolphinPartListingNotificationExtensionE_t>.strings))
        return static_cast<void*>(this);
    return KParts::ListingNotificationExtension::qt_metacast(_clname);
}

int DolphinPartListingNotificationExtension::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::ListingNotificationExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
