/****************************************************************************
** Meta object code from reading C++ file 'dolphintabwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.04.1/src/dolphintabwidget.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dolphintabwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16DolphinTabWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto DolphinTabWidget::qt_create_metaobjectdata<qt_meta_tag_ZN16DolphinTabWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DolphinTabWidget",
        "activeViewChanged",
        "",
        "DolphinViewContainer*",
        "viewContainer",
        "tabCountChanged",
        "count",
        "rememberClosedTab",
        "QUrl",
        "url",
        "state",
        "currentUrlChanged",
        "urlChanged",
        "openNewActivatedTab",
        "primaryUrl",
        "secondaryUrl",
        "openNewTab",
        "DolphinTabPage*",
        "DolphinTabWidget::NewTabPosition",
        "position",
        "openDirectories",
        "QList<QUrl>",
        "dirs",
        "splitView",
        "openFiles",
        "files",
        "closeTab",
        "index",
        "activateTab",
        "activateLastTab",
        "activateNextTab",
        "activatePrevTab",
        "restoreClosedTab",
        "copyToInactiveSplitView",
        "moveToInactiveSplitView",
        "detachTab",
        "tabDragMoveEvent",
        "tab",
        "QDragMoveEvent*",
        "event",
        "tabDropEvent",
        "QDropEvent*",
        "tabUrlChanged",
        "currentTabChanged",
        "renameTab",
        "label"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activeViewChanged'
        QtMocHelpers::SignalData<void(DolphinViewContainer *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'tabCountChanged'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'rememberClosedTab'
        QtMocHelpers::SignalData<void(const QUrl &, const QByteArray &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::QByteArray, 10 },
        }}),
        // Signal 'currentUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'urlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'openNewActivatedTab'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openNewActivatedTab'
        QtMocHelpers::SlotData<void(const QUrl &, const QUrl &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 14 }, { 0x80000000 | 8, 15 },
        }}),
        // Slot 'openNewActivatedTab'
        QtMocHelpers::SlotData<void(const QUrl &)>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 8, 14 },
        }}),
        // Slot 'openNewTab'
        QtMocHelpers::SlotData<DolphinTabPage *(const QUrl &, const QUrl &, DolphinTabWidget::NewTabPosition)>(16, 2, QMC::AccessPublic, 0x80000000 | 17, {{
            { 0x80000000 | 8, 14 }, { 0x80000000 | 8, 15 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'openNewTab'
        QtMocHelpers::SlotData<DolphinTabPage *(const QUrl &, const QUrl &)>(16, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 17, {{
            { 0x80000000 | 8, 14 }, { 0x80000000 | 8, 15 },
        }}),
        // Slot 'openNewTab'
        QtMocHelpers::SlotData<DolphinTabPage *(const QUrl &)>(16, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 17, {{
            { 0x80000000 | 8, 14 },
        }}),
        // Slot 'openDirectories'
        QtMocHelpers::SlotData<void(const QList<QUrl> &, bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::Bool, 23 },
        }}),
        // Slot 'openFiles'
        QtMocHelpers::SlotData<void(const QList<QUrl> &, bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 25 }, { QMetaType::Bool, 23 },
        }}),
        // Slot 'closeTab'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'closeTab'
        QtMocHelpers::SlotData<void(const int)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'activateTab'
        QtMocHelpers::SlotData<void(const int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'activateLastTab'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activateNextTab'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activatePrevTab'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'restoreClosedTab'
        QtMocHelpers::SlotData<void(const QByteArray &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 10 },
        }}),
        // Slot 'copyToInactiveSplitView'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveToInactiveSplitView'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'detachTab'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'openNewActivatedTab'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'tabDragMoveEvent'
        QtMocHelpers::SlotData<void(int, QDragMoveEvent *)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 37 }, { 0x80000000 | 38, 39 },
        }}),
        // Slot 'tabDropEvent'
        QtMocHelpers::SlotData<void(int, QDropEvent *)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 37 }, { 0x80000000 | 41, 39 },
        }}),
        // Slot 'tabUrlChanged'
        QtMocHelpers::SlotData<void(const QUrl &)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'currentTabChanged'
        QtMocHelpers::SlotData<void(int)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'renameTab'
        QtMocHelpers::SlotData<void(int, const QString &)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 }, { QMetaType::QString, 45 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DolphinTabWidget, qt_meta_tag_ZN16DolphinTabWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DolphinTabWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QTabWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DolphinTabWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DolphinTabWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16DolphinTabWidgetE_t>.metaTypes,
    nullptr
} };

void DolphinTabWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DolphinTabWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activeViewChanged((*reinterpret_cast<std::add_pointer_t<DolphinViewContainer*>>(_a[1]))); break;
        case 1: _t->tabCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->rememberClosedTab((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 3: _t->currentUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->urlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 5: _t->openNewActivatedTab(); break;
        case 6: _t->openNewActivatedTab((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 7: _t->openNewActivatedTab((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: { DolphinTabPage* _r = _t->openNewTab((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<DolphinTabWidget::NewTabPosition>>(_a[3])));
            if (_a[0]) *reinterpret_cast<DolphinTabPage**>(_a[0]) = std::move(_r); }  break;
        case 9: { DolphinTabPage* _r = _t->openNewTab((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])));
            if (_a[0]) *reinterpret_cast<DolphinTabPage**>(_a[0]) = std::move(_r); }  break;
        case 10: { DolphinTabPage* _r = _t->openNewTab((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<DolphinTabPage**>(_a[0]) = std::move(_r); }  break;
        case 11: _t->openDirectories((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 12: _t->openFiles((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 13: _t->closeTab(); break;
        case 14: _t->closeTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->activateTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->activateLastTab(); break;
        case 17: _t->activateNextTab(); break;
        case 18: _t->activatePrevTab(); break;
        case 19: _t->restoreClosedTab((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 20: _t->copyToInactiveSplitView(); break;
        case 21: _t->moveToInactiveSplitView(); break;
        case 22: _t->detachTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->openNewActivatedTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->tabDragMoveEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDragMoveEvent*>>(_a[2]))); break;
        case 25: _t->tabDropEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDropEvent*>>(_a[2]))); break;
        case 26: _t->tabUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 27: _t->currentTabChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->renameTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DolphinTabWidget::*)(DolphinViewContainer * )>(_a, &DolphinTabWidget::activeViewChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinTabWidget::*)(int )>(_a, &DolphinTabWidget::tabCountChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinTabWidget::*)(const QUrl & , const QByteArray & )>(_a, &DolphinTabWidget::rememberClosedTab, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinTabWidget::*)(const QUrl & )>(_a, &DolphinTabWidget::currentUrlChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinTabWidget::*)(const QUrl & )>(_a, &DolphinTabWidget::urlChanged, 4))
            return;
    }
}

const QMetaObject *DolphinTabWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DolphinTabWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DolphinTabWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QTabWidget::qt_metacast(_clname);
}

int DolphinTabWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTabWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 29;
    }
    return _id;
}

// SIGNAL 0
void DolphinTabWidget::activeViewChanged(DolphinViewContainer * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DolphinTabWidget::tabCountChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DolphinTabWidget::rememberClosedTab(const QUrl & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void DolphinTabWidget::currentUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DolphinTabWidget::urlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
