/****************************************************************************
** Meta object code from reading C++ file 'kitemlistcontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.04.1/src/kitemviews/kitemlistcontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kitemlistcontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19KItemListControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto KItemListController::qt_create_metaobjectdata<qt_meta_tag_ZN19KItemListControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItemListController",
        "itemActivated",
        "",
        "index",
        "itemsActivated",
        "KItemSet",
        "indexes",
        "itemMiddleClicked",
        "itemContextMenuRequested",
        "QPointF",
        "pos",
        "viewContextMenuRequested",
        "headerContextMenuRequested",
        "itemHovered",
        "itemUnhovered",
        "mouseButtonPressed",
        "itemIndex",
        "Qt::MouseButtons",
        "buttons",
        "mouseButtonReleased",
        "itemExpansionToggleClicked",
        "itemDropEvent",
        "QGraphicsSceneDragDropEvent*",
        "event",
        "aboveItemDropEvent",
        "escapePressed",
        "selectionModeChangeRequested",
        "enabled",
        "modelChanged",
        "KItemModelBase*",
        "current",
        "previous",
        "viewChanged",
        "KItemListView*",
        "selectedItemTextPressed",
        "scrollerStop",
        "increaseZoom",
        "decreaseZoom",
        "swipeUp",
        "doubleClickViewBackground",
        "Qt::MouseButton",
        "button",
        "slotStateChanged",
        "QScroller::State",
        "newState",
        "slotViewScrollOffsetChanged",
        "slotRubberBandChanged",
        "slotChangeCurrentItem",
        "text",
        "searchFromNextItem",
        "bool*",
        "found",
        "slotAutoActivationTimeout",
        "model",
        "view",
        "SelectionBehavior",
        "NoSelection",
        "SingleSelection",
        "MultiSelection"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemActivated'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'itemsActivated'
        QtMocHelpers::SignalData<void(const KItemSet &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'itemMiddleClicked'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'itemContextMenuRequested'
        QtMocHelpers::SignalData<void(int, const QPointF &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'viewContextMenuRequested'
        QtMocHelpers::SignalData<void(const QPointF &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'headerContextMenuRequested'
        QtMocHelpers::SignalData<void(const QPointF &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'itemHovered'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'itemUnhovered'
        QtMocHelpers::SignalData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'mouseButtonPressed'
        QtMocHelpers::SignalData<void(int, Qt::MouseButtons)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Signal 'mouseButtonReleased'
        QtMocHelpers::SignalData<void(int, Qt::MouseButtons)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Signal 'itemExpansionToggleClicked'
        QtMocHelpers::SignalData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'itemDropEvent'
        QtMocHelpers::SignalData<void(int, QGraphicsSceneDragDropEvent *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 22, 23 },
        }}),
        // Signal 'aboveItemDropEvent'
        QtMocHelpers::SignalData<void(int, QGraphicsSceneDragDropEvent *)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 22, 23 },
        }}),
        // Signal 'escapePressed'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectionModeChangeRequested'
        QtMocHelpers::SignalData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Signal 'modelChanged'
        QtMocHelpers::SignalData<void(KItemModelBase *, KItemModelBase *)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { 0x80000000 | 29, 31 },
        }}),
        // Signal 'viewChanged'
        QtMocHelpers::SignalData<void(KItemListView *, KItemListView *)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 30 }, { 0x80000000 | 33, 31 },
        }}),
        // Signal 'selectedItemTextPressed'
        QtMocHelpers::SignalData<void(int)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'scrollerStop'
        QtMocHelpers::SignalData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'increaseZoom'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'decreaseZoom'
        QtMocHelpers::SignalData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'swipeUp'
        QtMocHelpers::SignalData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'doubleClickViewBackground'
        QtMocHelpers::SignalData<void(Qt::MouseButton)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 41 },
        }}),
        // Slot 'slotStateChanged'
        QtMocHelpers::SlotData<void(QScroller::State)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 43, 44 },
        }}),
        // Slot 'slotViewScrollOffsetChanged'
        QtMocHelpers::SlotData<void(qreal, qreal)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QReal, 30 }, { QMetaType::QReal, 31 },
        }}),
        // Slot 'slotRubberBandChanged'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotChangeCurrentItem'
        QtMocHelpers::SlotData<void(const QString &, bool, bool *)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 48 }, { QMetaType::Bool, 49 }, { 0x80000000 | 50, 51 },
        }}),
        // Slot 'slotAutoActivationTimeout'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'model'
        QtMocHelpers::PropertyData<KItemModelBase*>(53, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 15),
        // property 'view'
        QtMocHelpers::PropertyData<KItemListView*>(54, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 16),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SelectionBehavior'
        QtMocHelpers::EnumData<enum SelectionBehavior>(55, 55, QMC::EnumFlags{}).add({
            {   56, SelectionBehavior::NoSelection },
            {   57, SelectionBehavior::SingleSelection },
            {   58, SelectionBehavior::MultiSelection },
        }),
    };
    return QtMocHelpers::metaObjectData<KItemListController, qt_meta_tag_ZN19KItemListControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItemListController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KItemListControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KItemListControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19KItemListControllerE_t>.metaTypes,
    nullptr
} };

void KItemListController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KItemListController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->itemsActivated((*reinterpret_cast<std::add_pointer_t<KItemSet>>(_a[1]))); break;
        case 2: _t->itemMiddleClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->itemContextMenuRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 4: _t->viewContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 5: _t->headerContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 6: _t->itemHovered((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->itemUnhovered((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->mouseButtonPressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::MouseButtons>>(_a[2]))); break;
        case 9: _t->mouseButtonReleased((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::MouseButtons>>(_a[2]))); break;
        case 10: _t->itemExpansionToggleClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->itemDropEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QGraphicsSceneDragDropEvent*>>(_a[2]))); break;
        case 12: _t->aboveItemDropEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QGraphicsSceneDragDropEvent*>>(_a[2]))); break;
        case 13: _t->escapePressed(); break;
        case 14: _t->selectionModeChangeRequested((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->modelChanged((*reinterpret_cast<std::add_pointer_t<KItemModelBase*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KItemModelBase*>>(_a[2]))); break;
        case 16: _t->viewChanged((*reinterpret_cast<std::add_pointer_t<KItemListView*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KItemListView*>>(_a[2]))); break;
        case 17: _t->selectedItemTextPressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->scrollerStop(); break;
        case 19: _t->increaseZoom(); break;
        case 20: _t->decreaseZoom(); break;
        case 21: _t->swipeUp(); break;
        case 22: _t->doubleClickViewBackground((*reinterpret_cast<std::add_pointer_t<Qt::MouseButton>>(_a[1]))); break;
        case 23: _t->slotStateChanged((*reinterpret_cast<std::add_pointer_t<QScroller::State>>(_a[1]))); break;
        case 24: _t->slotViewScrollOffsetChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 25: _t->slotRubberBandChanged(); break;
        case 26: _t->slotChangeCurrentItem((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool*>>(_a[3]))); break;
        case 27: _t->slotAutoActivationTimeout(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int )>(_a, &KItemListController::itemActivated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(const KItemSet & )>(_a, &KItemListController::itemsActivated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int )>(_a, &KItemListController::itemMiddleClicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int , const QPointF & )>(_a, &KItemListController::itemContextMenuRequested, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(const QPointF & )>(_a, &KItemListController::viewContextMenuRequested, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(const QPointF & )>(_a, &KItemListController::headerContextMenuRequested, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int )>(_a, &KItemListController::itemHovered, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int )>(_a, &KItemListController::itemUnhovered, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int , Qt::MouseButtons )>(_a, &KItemListController::mouseButtonPressed, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int , Qt::MouseButtons )>(_a, &KItemListController::mouseButtonReleased, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int )>(_a, &KItemListController::itemExpansionToggleClicked, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int , QGraphicsSceneDragDropEvent * )>(_a, &KItemListController::itemDropEvent, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int , QGraphicsSceneDragDropEvent * )>(_a, &KItemListController::aboveItemDropEvent, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)()>(_a, &KItemListController::escapePressed, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(bool )>(_a, &KItemListController::selectionModeChangeRequested, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(KItemModelBase * , KItemModelBase * )>(_a, &KItemListController::modelChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(KItemListView * , KItemListView * )>(_a, &KItemListController::viewChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(int )>(_a, &KItemListController::selectedItemTextPressed, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)()>(_a, &KItemListController::scrollerStop, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)()>(_a, &KItemListController::increaseZoom, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)()>(_a, &KItemListController::decreaseZoom, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)()>(_a, &KItemListController::swipeUp, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListController::*)(Qt::MouseButton )>(_a, &KItemListController::doubleClickViewBackground, 22))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KItemModelBase**>(_v) = _t->model(); break;
        case 1: *reinterpret_cast<KItemListView**>(_v) = _t->view(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setModel(*reinterpret_cast<KItemModelBase**>(_v)); break;
        case 1: _t->setView(*reinterpret_cast<KItemListView**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KItemListController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KItemListController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KItemListControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KItemListController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 28;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KItemListController::itemActivated(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KItemListController::itemsActivated(const KItemSet & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KItemListController::itemMiddleClicked(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KItemListController::itemContextMenuRequested(int _t1, const QPointF & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void KItemListController::viewContextMenuRequested(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KItemListController::headerContextMenuRequested(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KItemListController::itemHovered(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void KItemListController::itemUnhovered(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void KItemListController::mouseButtonPressed(int _t1, Qt::MouseButtons _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void KItemListController::mouseButtonReleased(int _t1, Qt::MouseButtons _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void KItemListController::itemExpansionToggleClicked(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void KItemListController::itemDropEvent(int _t1, QGraphicsSceneDragDropEvent * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void KItemListController::aboveItemDropEvent(int _t1, QGraphicsSceneDragDropEvent * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void KItemListController::escapePressed()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void KItemListController::selectionModeChangeRequested(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void KItemListController::modelChanged(KItemModelBase * _t1, KItemModelBase * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1, _t2);
}

// SIGNAL 16
void KItemListController::viewChanged(KItemListView * _t1, KItemListView * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void KItemListController::selectedItemTextPressed(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void KItemListController::scrollerStop()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void KItemListController::increaseZoom()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void KItemListController::decreaseZoom()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void KItemListController::swipeUp()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void KItemListController::doubleClickViewBackground(Qt::MouseButton _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}
QT_WARNING_POP
