/****************************************************************************
** Meta object code from reading C++ file 'kitemmodelbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.04.1/src/kitemviews/kitemmodelbase.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kitemmodelbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14KItemModelBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto KItemModelBase::qt_create_metaobjectdata<qt_meta_tag_ZN14KItemModelBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItemModelBase",
        "itemsInserted",
        "",
        "KItemRangeList",
        "itemRanges",
        "itemsRemoved",
        "itemsMoved",
        "KItemRange",
        "itemRange",
        "QList<int>",
        "movedToIndexes",
        "itemsChanged",
        "QSet<QByteArray>",
        "roles",
        "groupsChanged",
        "groupedSortingChanged",
        "current",
        "sortRoleChanged",
        "previous",
        "sortOrderChanged",
        "Qt::SortOrder"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemsInserted'
        QtMocHelpers::SignalData<void(const KItemRangeList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'itemsRemoved'
        QtMocHelpers::SignalData<void(const KItemRangeList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'itemsMoved'
        QtMocHelpers::SignalData<void(const KItemRange &, const QList<int> &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'itemsChanged'
        QtMocHelpers::SignalData<void(const KItemRangeList &, const QSet<QByteArray> &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 12, 13 },
        }}),
        // Signal 'groupsChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'groupedSortingChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Signal 'sortRoleChanged'
        QtMocHelpers::SignalData<void(const QByteArray &, const QByteArray &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 16 }, { QMetaType::QByteArray, 18 },
        }}),
        // Signal 'sortOrderChanged'
        QtMocHelpers::SignalData<void(Qt::SortOrder, Qt::SortOrder)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 16 }, { 0x80000000 | 20, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KItemModelBase, qt_meta_tag_ZN14KItemModelBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItemModelBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KItemModelBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KItemModelBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14KItemModelBaseE_t>.metaTypes,
    nullptr
} };

void KItemModelBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KItemModelBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemsInserted((*reinterpret_cast<std::add_pointer_t<KItemRangeList>>(_a[1]))); break;
        case 1: _t->itemsRemoved((*reinterpret_cast<std::add_pointer_t<KItemRangeList>>(_a[1]))); break;
        case 2: _t->itemsMoved((*reinterpret_cast<std::add_pointer_t<KItemRange>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2]))); break;
        case 3: _t->itemsChanged((*reinterpret_cast<std::add_pointer_t<KItemRangeList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSet<QByteArray>>>(_a[2]))); break;
        case 4: _t->groupsChanged(); break;
        case 5: _t->groupedSortingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->sortRoleChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 7: _t->sortOrderChanged((*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QByteArray> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KItemModelBase::*)(const KItemRangeList & )>(_a, &KItemModelBase::itemsInserted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemModelBase::*)(const KItemRangeList & )>(_a, &KItemModelBase::itemsRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemModelBase::*)(const KItemRange & , const QList<int> & )>(_a, &KItemModelBase::itemsMoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemModelBase::*)(const KItemRangeList & , const QSet<QByteArray> & )>(_a, &KItemModelBase::itemsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemModelBase::*)()>(_a, &KItemModelBase::groupsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemModelBase::*)(bool )>(_a, &KItemModelBase::groupedSortingChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemModelBase::*)(const QByteArray & , const QByteArray & )>(_a, &KItemModelBase::sortRoleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemModelBase::*)(Qt::SortOrder , Qt::SortOrder )>(_a, &KItemModelBase::sortOrderChanged, 7))
            return;
    }
}

const QMetaObject *KItemModelBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KItemModelBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KItemModelBaseE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KItemModelBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void KItemModelBase::itemsInserted(const KItemRangeList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KItemModelBase::itemsRemoved(const KItemRangeList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KItemModelBase::itemsMoved(const KItemRange & _t1, const QList<int> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void KItemModelBase::itemsChanged(const KItemRangeList & _t1, const QSet<QByteArray> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void KItemModelBase::groupsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KItemModelBase::groupedSortingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KItemModelBase::sortRoleChanged(const QByteArray & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void KItemModelBase::sortOrderChanged(Qt::SortOrder _t1, Qt::SortOrder _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}
QT_WARNING_POP
