/****************************************************************************
** Meta object code from reading C++ file 'dolphinstatusbar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.04.1/src/statusbar/dolphinstatusbar.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dolphinstatusbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16DolphinStatusBarE_t {};
} // unnamed namespace

template <> constexpr inline auto DolphinStatusBar::qt_create_metaobjectdata<qt_meta_tag_ZN16DolphinStatusBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DolphinStatusBar",
        "stopPressed",
        "",
        "zoomLevelChanged",
        "zoomLevel",
        "showMessage",
        "message",
        "KMessageWidget::MessageType",
        "messageType",
        "modeUpdated",
        "widthUpdated",
        "urlChanged",
        "setText",
        "text",
        "setUrl",
        "QUrl",
        "url",
        "setZoomLevel",
        "showZoomSliderToolTip",
        "updateProgressInfo",
        "updateLabelText",
        "updateZoomSliderToolTip"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stopPressed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zoomLevelChanged'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'showMessage'
        QtMocHelpers::SignalData<void(const QString &, KMessageWidget::MessageType)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'modeUpdated'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'widthUpdated'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'urlChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setText'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'setUrl'
        QtMocHelpers::SlotData<void(const QUrl &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'setZoomLevel'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'showZoomSliderToolTip'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'updateProgressInfo'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateLabelText'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateZoomSliderToolTip'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DolphinStatusBar, qt_meta_tag_ZN16DolphinStatusBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DolphinStatusBar::staticMetaObject = { {
    QMetaObject::SuperData::link<AnimatedHeightWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DolphinStatusBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DolphinStatusBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16DolphinStatusBarE_t>.metaTypes,
    nullptr
} };

void DolphinStatusBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DolphinStatusBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stopPressed(); break;
        case 1: _t->zoomLevelChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->showMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KMessageWidget::MessageType>>(_a[2]))); break;
        case 3: _t->modeUpdated(); break;
        case 4: _t->widthUpdated(); break;
        case 5: _t->urlChanged(); break;
        case 6: _t->setText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->setUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: _t->setZoomLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->showZoomSliderToolTip((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->updateProgressInfo(); break;
        case 11: _t->updateLabelText(); break;
        case 12: _t->updateZoomSliderToolTip((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DolphinStatusBar::*)()>(_a, &DolphinStatusBar::stopPressed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinStatusBar::*)(int )>(_a, &DolphinStatusBar::zoomLevelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinStatusBar::*)(const QString & , KMessageWidget::MessageType )>(_a, &DolphinStatusBar::showMessage, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinStatusBar::*)()>(_a, &DolphinStatusBar::modeUpdated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinStatusBar::*)()>(_a, &DolphinStatusBar::widthUpdated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinStatusBar::*)()>(_a, &DolphinStatusBar::urlChanged, 5))
            return;
    }
}

const QMetaObject *DolphinStatusBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DolphinStatusBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DolphinStatusBarE_t>.strings))
        return static_cast<void*>(this);
    return AnimatedHeightWidget::qt_metacast(_clname);
}

int DolphinStatusBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AnimatedHeightWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void DolphinStatusBar::stopPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DolphinStatusBar::zoomLevelChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DolphinStatusBar::showMessage(const QString & _t1, KMessageWidget::MessageType _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void DolphinStatusBar::modeUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DolphinStatusBar::widthUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DolphinStatusBar::urlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
