// This file is generated by kconfig_compiler_kf6 from dolphin_folderspanelsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef FOLDERSPANELSETTINGS_H
#define FOLDERSPANELSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class FoldersPanelSettings : public KConfigSkeleton
{
  public:

    static FoldersPanelSettings *self();
    ~FoldersPanelSettings() override;

    /**
      Set Hidden files shown
    */
    static
    void setHiddenFilesShown( bool v )
    {
      if (!self()->isHiddenFilesShownImmutable())
        self()->mHiddenFilesShown = v;
    }

    /**
      Get Hidden files shown
    */
    static
    bool hiddenFilesShown()
    {
      return self()->mHiddenFilesShown;
    }

    /**
      Is Hidden files shown Immutable
    */
    static
    bool isHiddenFilesShownImmutable()
    {
      return self()->isImmutable( QStringLiteral( "HiddenFilesShown" ) );
    }

    /**
      Set Limit folders panel to home directory if inside home
    */
    static
    void setLimitFoldersPanelToHome( bool v )
    {
      if (!self()->isLimitFoldersPanelToHomeImmutable())
        self()->mLimitFoldersPanelToHome = v;
    }

    /**
      Get Limit folders panel to home directory if inside home
    */
    static
    bool limitFoldersPanelToHome()
    {
      return self()->mLimitFoldersPanelToHome;
    }

    /**
      Is Limit folders panel to home directory if inside home Immutable
    */
    static
    bool isLimitFoldersPanelToHomeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "LimitFoldersPanelToHome" ) );
    }

    /**
      Set Automatic scrolling
    */
    static
    void setAutoScrolling( bool v )
    {
      if (!self()->isAutoScrollingImmutable())
        self()->mAutoScrolling = v;
    }

    /**
      Get Automatic scrolling
    */
    static
    bool autoScrolling()
    {
      return self()->mAutoScrolling;
    }

    /**
      Is Automatic scrolling Immutable
    */
    static
    bool isAutoScrollingImmutable()
    {
      return self()->isImmutable( QStringLiteral( "AutoScrolling" ) );
    }

  protected:
    FoldersPanelSettings();
    friend class FoldersPanelSettingsHelper;


    // FoldersPanel
    bool mHiddenFilesShown;
    bool mLimitFoldersPanelToHome;
    bool mAutoScrolling;
};

#endif

