// This file is generated by kconfig_compiler_kf6 from dolphin_placespanelsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef PLACESPANELSETTINGS_H
#define PLACESPANELSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

#include <kiconloader.h>

class PlacesPanelSettings : public KConfigSkeleton
{
  public:

    static PlacesPanelSettings *self();
    ~PlacesPanelSettings() override;

    /**
      Set Size of icons in the Places Panel (-1 means "automatic")
    */
    static
    void setIconSize( int v )
    {
      if (!self()->isIconSizeImmutable())
        self()->mIconSize = v;
    }

    /**
      Get Size of icons in the Places Panel (-1 means "automatic")
    */
    static
    int iconSize()
    {
      return self()->mIconSize;
    }

    /**
      Is Size of icons in the Places Panel (-1 means "automatic") Immutable
    */
    static
    bool isIconSizeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "IconSize" ) );
    }

  protected:
    PlacesPanelSettings();
    friend class PlacesPanelSettingsHelper;


    // PlacesPanel
    int mIconSize;
};

#endif

